﻿using SharpChat.Events;
using SharpChat.Packet;
using System;
using System.Linq;

namespace SharpChat.Commands {
    public class WhisperCommand : IChatCommand {
        public bool IsMatch(ChatCommandContext ctx) {
            return ctx.NameEquals("whisper")
                || ctx.NameEquals("msg");
        }

        public void Dispatch(ChatCommandContext ctx) {
            if(ctx.Args.Length < 2) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.COMMAND_FORMAT_ERROR));
                return;
            }

            string whisperUserStr = ctx.Args.FirstOrDefault();
            ChatUser whisperUser = ctx.Chat.Users.FirstOrDefault(u => u.NameEquals(whisperUserStr));

            if(whisperUser == null) {
                ctx.Chat.SendTo(ctx.User, new LegacyCommandResponse(LCR.USER_NOT_FOUND, true, whisperUserStr));
                return;
            }

            if(whisperUser == ctx.User)
                return;

            ctx.Chat.DispatchEvent(new MessageCreateEvent(
                SharpId.Next(),
                ChatUser.GetDMChannelName(ctx.User, whisperUser),
                ctx.User,
                DateTimeOffset.Now,
                string.Join(' ', ctx.Args.Skip(1)),
                true, false, false
            ));
        }
    }
}
